#ifndef BisqPAPU_h
#define BisqPAPU_h

#include <cstdio>
#include <unistd.h>
#include "cpu.hh"
#include "papu_internals.hh"

#define PAPU_FIXEDPOINT 16 // BITS

class bisqpapu
{
    int fd;
    int Rate;
    
    sq1gen chan1;
    sq2gen chan2;
    papuhzticker<triangleunit> chan3;
    papuhzticker<randgenunit> chan4;
    dmcgen chan5;
    
    unsigned long counter, speed;
    
    void mixbytes(unsigned count);
    
public:
    bisqpapu();
    ~bisqpapu();
    
    bool write(const fWORD addr, fBYTE value);
    bool read(const fWORD addr, BYTE &value);
    
    bool tick()
    {
        // This function runs Clock->ClockHz / Clock->CPUclockDivider
        // times per second. For NTSC, it is 1.7897725 MHz.

        chan1.tick();
        chan2.tick();
        chan3.tick();
        chan4.tick();
        bool irq = chan5.tick();
        
        counter += speed;
        if(counter >= (1 << PAPU_FIXEDPOINT))
        {
            mixbytes(counter >> PAPU_FIXEDPOINT);
            counter &= (1 << PAPU_FIXEDPOINT)-1;
        }

        return irq;
    }
    
    void debug_silence()
    {
    	fprintf(stderr, "\33[1;36mpAPU channels:\n");
    	fprintf(stderr, " 1: "); chan1.debug_silence(); fprintf(stderr, "\n");
    	fprintf(stderr, " 2: "); chan2.debug_silence(); fprintf(stderr, "\n");
    	fprintf(stderr, " 3: "); chan3.debug_silence(); fprintf(stderr, "\n");
    	fprintf(stderr, " 4: "); chan4.debug_silence(); fprintf(stderr, "\n");
    	fprintf(stderr, " 5: "); chan5.debug_silence(); fprintf(stderr, "\n");
		fprintf(stderr, "\33[m\n");
    }
};

#endif
